#!/usr/bin/env Rscript
source("predict_wrapper.R")

args = commandArgs(trailingOnly=TRUE)

Ncore = 1
Nrep = 1
Nbg = 1

remove_dups = F
return_preds=F
matchCLsAllDataTypes = T # only use cell lines with genomic+transcriptomic+proteomic data

ALLOWED_DATATYPES = c("rppa","rnaseq")
ALLOWED_TISSUES = c("kidney", "liver", "urinarytract", "stomach", "oesophagus", "endometrium", "upperaerodigestivetract", "largeintestine", "centralnervoussystem", "breast", "pancreas", "ovary", "skin", "haematopoieticandlymphoidtissue", "lung", "all")
postfix=""

method="pls"

if(length(args)==4) {
  tissue = args[1]
  dt = args[2]
  sen = args[3]
  num = as.numeric(args[4])
  postfix = paste("_",num,sep="")
  if(!(tissue %in% ALLOWED_TISSUES)) {
    print("Unknown tissue, quitting.")
    return()
  } 
  if(!(dt %in% ALLOWED_DATATYPES)) {
    print("Unknown datatype, quitting.")
    return()
  }

} else {
  print("Wrong number of arguments")
  return()
}

if(dt == "rppa") {
	filt_settings = list(mean_filt=-999, sd_filt=0.1)
	dataX_within =  list(datasource="ccle", datatype=dt, tissue=tissue)
	dataX_cross = list(datasource="ccle", datatype=dt)
} else if(dt == "rnaseq") {
	filt_settings = list(mean_filt=-999, sd_filt=0.1)
	dataX_within =  list(datasource="ccle", datatype=dt, tissue=tissue, version="cosmic")
	dataX_cross =  list(datasource="ccle", datatype=dt, version="cosmic")
}

dataY_within = list(datasource=sen, datatype="sen", tissue=tissue)
dataY_cross = list(datasource=sen, datatype="sen")

filename = paste("analysis/cross_", tissue,"_", dt, "_", sen,"_",num,".RData", sep="")
df = data.frame()
df_fair = data.frame()

if(matchCLsAllDataTypes) {
  matchCLsAllDataTypesList = list(list(datasource="ccle", datatype="mut"),
                               list(datasource="ccle", datatype="rnaseq"),
                               list(datasource="ccle", datatype="rppa"))
} else {
  matchCLsAllDataTypesList = NULL
}

cls = rownames(getData(dataX_cross, match_CLs = c(list(dataY_cross), matchCLsAllDataTypesList)))
tissue_data = getData(dataX_within, match_CLs = c(list(dataY_within), matchCLsAllDataTypesList))
tissue_cls = rownames(tissue_data)
tissue_sen = getData(dataY_within, match_CLs = c(list(dataX_within), matchCLsAllDataTypesList))
        
out_within = predict_wrapper(dataX=dataX_within, 
                        dataY=dataY_within, 
                        match_CLs = matchCLsAllDataTypesList,
                        Nrep=1, Nbg=0, Ncore=1, method=method, return_model = T, 
                        remove_dups = remove_dups, filt_settings=filt_settings)
        
if(is.null(out_within)) {
    print(paste(tissue, dt, sen, num, "WITHIN-IS-NULL", sep=","))
    save(df, df_fair, file = filename)
    quit()
}

print("Finished: out_within")
        
out_cross = predict_wrapper(dataX=dataX_cross, 
                       dataY=dataY_cross,
                       match_CLs = matchCLsAllDataTypesList,
                       exclude_CLs = tissue_cls, 
                       Nrep=1, Nbg=0, Ncore=1, method=method, filt_settings=filt_settings,
                       return_model = T, remove_dups = remove_dups)

if(is.null(out_cross)) {
    print(paste(tissue,	dt, sen, num, "CROSS-IS-NULL", sep=","))
    save(df, df_fair, file = filename)
    quit()
}

print("Finished: out_cross")
        
# fair = predict from the same number of CLs within tissue as cross tissue
cls_fair = sample(setdiff(cls, tissue_cls), length(tissue_cls))
cls_fair_exclude = setdiff(cls, cls_fair)

out_cross_fair = predict_wrapper(dataX=dataX_cross, 
			    dataY=dataY_cross, 
			    match_CLs = matchCLsAllDataTypesList,
			    exclude_CLs = cls_fair_exclude, filt_settings=filt_settings, 
			    Nrep=1, Nbg=0, Ncore=1, method=method, 
			    return_model = T, remove_dups = remove_dups)
if(is.null(out_cross_fair)) {
    print(paste(tissue,	dt, sen, num, "CROSS-FAIR-IS-NULL", sep=","))
    save(df, df_fair, file = filename)
    quit()
}

print("Finished: out_cross_fair")

    
cross_pred = function(out_cross, drugs) {
	ret=data.frame()
        for(drug in intersect(drugs,names(out_within$cors))) {
            
        	outX = names(out_cross$model[[drug]]$coefficients[,1,1])
        	drug_ncomp = paste(drug,"_ncomp",sep="")
        	ncomp = out_cross$models[[paste(drug,"_ncomp",sep="")]]
            
        	if(ncomp==0)
            		ncomp=1
            
        	tissue_data_filt = tissue_data[,outX]
        	preds_cross = predict(out_cross$models[[drug]], ncomp=ncomp, newdata=as.matrix(tissue_data_filt))
        	preds_cross = preds_cross[,1,1]
            
        	meas = tissue_sen[,drug]
        	names(meas) = rownames(tissue_sen[,drug,drop=F])
            
        	common_cls = intersect(names(preds_cross)[!is.na(names(preds_cross))], 
                                 names(meas)[!is.na(names(meas))] )
            
        	preds_cross = preds_cross[common_cls]
        	meas = meas[common_cls]
            
        	if(any(rownames(tissue_sen) != rownames(preds_cross))) {
           		print("ERROR: Not matching CLs between predictions and measured sensitivity")
           		break
        	}   

        	tmp = tibble(datatype=dt, 
                     tissue=tissue,
                     sen=sen,
                     drug = drug,
                     drug_target = out_within$drug_targets[drug], 
                     cor_cross = cor(preds_cross, meas, method = "spearman", use="complete.obs"),
                     cor_within = out_within$cors[drug],
                     varExpl_cross = R2(preds_cross, meas),
                     varExpl_within = out_within$varsExpl[drug])
            
        	ret = rbind(ret, tmp)
            
        }
        return(ret)
}
        
drugs = names(out_cross$models)[!grepl("_ncomp", names(out_cross$models))]
drugs_fair = names(out_cross_fair$models)[!grepl("_ncomp", names(out_cross_fair$models))]
        
df = cross_pred(out_cross, drugs)
df_fair = cross_pred(out_cross_fair, drugs_fair)

save(df, df_fair, file = filename)

a=1
b=2
c=a+b

