options(stringsAsFactors=FALSE)
library(tidyverse)
library(scales)
library(ggdendro)
library(cowplot)

source(file="settings.R")

####################################
###### PLOT VIA-PREDICTION #########
####################################
plot_predict = function(input, savepdf_cors=F, savepdf_varsExpl=F, savepdf_cors_bg=F, savepdf_varsExpl_bg=F, plot_title=F, p_pos="ll", manual_cont_annotation=NULL, manual_cont_annotation_label=NULL) {
  cors = input$cors
  varsExpl = input$varsExpl
  plot_errorbars = 'cors_rep' %in% names(input)
  plot_bg = 'cors_bg' %in% names(input)
  x_names = names(cors)
  has_drug_targets = 'drug_targets' %in% names(input)
  has_manual_cont_annotation = !is.null(manual_cont_annotation)
  
  title = input$argstr
  title_len = str_length(title)
  if(title_len>50)
    title = paste(str_sub(title, 1, floor(title_len/2)),"\n",str_sub(title, floor(title_len/2)+1, title_len), sep="")
  
  if(has_drug_targets) {
    drug_targets = input$drug_targets
    drug_type_lab = drug_type_label(drug_targets)
    drug_targets = drug_type_lab$drug_targets
    drug_target_color = drug_type_lab$drug_target_color
  }
  
  if(has_manual_cont_annotation) {
    manual_cont_annotation = manual_cont_annotation[names(manual_cont_annotation) %in% x_names]
    df_manual_cont_annotation = data.frame(x_names=names(manual_cont_annotation), val=manual_cont_annotation)
    missing_x_names = setdiff(x_names,names(manual_cont_annotation))
    if(length(missing_x_names)>0)
      df_manual_cont_annotation = rbind(df_manual_cont_annotation, 
                                        data.frame(x_names=missing_x_names,val=0))
  }
  
  
  if(plot_bg) {
    cors_bg = input$cors_bg
    varsExpl_bg = input$varsExpl_bg
    pvals_cors = input$pvals_cors
    pvals_varsExpl = input$pvals_varsExpl
    
    druglist_cors_bg = rep(x_names,each = nrow(cors_bg))
    druglist_cors_bg=ordered(druglist_cors_bg,levels=x_names[order(cors,decreasing=T)])
    df_cors_bg = data.frame(x_names=druglist_cors_bg,cors=matlab::reshape(cors_bg,nrow(cors_bg)*ncol(cors_bg),1))
    
    druglist_varsExpl_bg = rep(x_names,each = nrow(varsExpl_bg))
    druglist_varsExpl_bg=ordered(druglist_varsExpl_bg,levels=x_names[order(varsExpl,decreasing=T)])
    df_varsExpl_bg = data.frame(x_names=druglist_varsExpl_bg,varsExpl=matlab::reshape(varsExpl_bg,nrow(varsExpl_bg)*ncol(varsExpl_bg),1))
    
  }
  
  if(plot_errorbars) {
    errorbar_cors = input$cors_rep
    errorbar_varsExpl = input$varsExpl_rep
    
    druglist_cors_errorbar = rep(x_names,each = nrow(errorbar_cors))
    druglist_cors_errorbar=ordered(druglist_cors_errorbar,levels=x_names[order(cors,decreasing=T)])
    df_cors_errorbar = data.frame(x_names=druglist_cors_errorbar,cors=matlab::reshape(errorbar_cors,nrow(errorbar_cors)*ncol(errorbar_cors),1))
    
    druglist_varsExpl_errorbar = rep(x_names,each = nrow(errorbar_varsExpl))
    druglist_varsExpl_errorbar = ordered(druglist_varsExpl_errorbar,levels=x_names[order(varsExpl,decreasing=T)])
    df_varsExpl_errorbar = data.frame(x_names=druglist_varsExpl_errorbar,varsExpl=matlab::reshape(errorbar_varsExpl, nrow(errorbar_varsExpl)*ncol(errorbar_varsExpl),1))
  }
  
  quantiles_se <- function(x) {
    r <- c(mean(x)-sd(x), mean(x)+sd(x) )
    names(r) <- c("ymin", "ymax")
    r
  }
  
  errorbar_fn <- function(x) {
    r = c(mean(x)-sd(x),mean(x)+sd(x))
    names(r) = c("ymin","ymax")
    r
  }
  
  df_cors = data.frame(x_names=ordered(x_names,levels=x_names[order(cors,decreasing=T)]),cors=cors) # pvals=-log10(pvals_cors)
  df_varsExpl = data.frame(x_names=ordered(x_names,levels=x_names[order(varsExpl,decreasing=T)]),varsExpl=varsExpl) #,pvals=-log10(pvals_varsExpl)
  if(plot_bg) {
    df_cors$pvals = -log10(pvals_cors)
    df_varsExpl$pvals = -log10(pvals_varsExpl)
  }
  
  ret = list()
  
  gg_cors <- ggplot(df_cors, aes(x = x_names, y=cors))+
    geom_point()+
    xlab("")+
    ylab("Spearman rank correlation")+
    theme(axis.text.x = element_text(colour = "black",size=size_axes_text,angle=-90,hjust=0,vjust=0.5),
          axis.text.y = element_text(colour = "black",size=size_axes_text),
          axis.title = element_text(size=size_axes_title),
          legend.key = element_blank(),
          panel.grid.major = element_blank(), 
          panel.grid.minor = element_blank(),
          panel.background = element_rect(colour = "black", fill=NA, size=size_panel_border, linetype="solid"))
  
  gg_varsExpl <- ggplot(df_varsExpl, aes(x = x_names, y=varsExpl))+
    geom_point()+
    xlab("")+
    labs(fill="Drug type")+
    ylab("Variance explained")+
    theme(axis.text.x = element_text(colour = "black", size=size_axes_text, angle=-90,hjust=0, vjust=0.5),
          axis.text.y = element_text(colour = "black", size=size_axes_text),
          axis.title = element_text(size=size_axes_title),
          legend.key = element_blank(),
          panel.grid.major = element_blank(), 
          panel.grid.minor = element_blank(),
          panel.background = element_rect(colour = "black", fill=NA, size=size_panel_border, linetype="solid"))
  
  if(plot_title) {
    gg_cors <- gg_cors + ggtitle(title) + theme(plot.title = element_text(size = 5))
    gg_varsExpl <- gg_varsExpl + ggtitle(title) + theme(plot.title = element_text(size = 5))
  }  
  
  if(plot_bg) {
    
    ymax_mean_cors = df_cors_bg %>% 
      group_by(x_names) %>% 
      summarise(ymax=quantile(cors, probs=0.95))
    
    ymax_mean_varsExpl = df_varsExpl_bg %>% 
      group_by(x_names) %>% 
      summarise(ymax=quantile(varsExpl, probs=0.95))
    
    ymin_cors = 1.05*min(df_cors$cors, min(apply(cors_bg,2, function(x) quantiles_95(x)[1] )))-0.10
    ymax_cors = 1.07*max(df_cors$cors, max(apply(cors_bg,2, function(x) quantiles_95(x)[length(quantiles_95(x))] )))
    ymin_varsExpl = 1.05*min(df_varsExpl[,2], min(apply(varsExpl_bg,2, function(x) quantiles_95(x)[1] )))-0.1
    ymax_varsExpl = 1.07*max(df_varsExpl[,2], max(apply(varsExpl_bg,2, function(x) quantiles_95(x)[length(quantiles_95(x))] )))
    
    gg_cors_bg <- gg_cors+ 
      stat_summary(data=df_cors_bg, aes(x=x_names, y=cors), fun.data = quantiles_95, geom="boxplot",fill="grey80", position="dodge" )+
      geom_point(data=df_cors,aes(x=x_names,y=cors,colour=pvals),size=size_marker)+
      
      scale_colour_gradient(expression(-log[10](P)),high="red",low="blue",limits=c(0,4))+
      coord_cartesian(ylim=c(ymin_cors, ymax_cors))+
      guides(color=guide_colorbar(title.theme = element_text(size=size_legend, angle=0),
                                  label.theme = element_text(size=size_legend, angle=0)))
    
    gg_varsExpl_bg <- gg_varsExpl+ 
      stat_summary(data=df_varsExpl_bg, aes(x=x_names, y=varsExpl), fun.data = quantiles_95, geom="boxplot",fill="grey80", position="dodge" )+
      geom_point(data=df_varsExpl,aes(x=x_names,y=varsExpl,colour=pvals),size=size_marker)+
      scale_colour_gradient(expression(-log[10](P)),high="red",low="blue",limits=c(0,4), breaks=c(0,2,4))+
      coord_cartesian(ylim=c(ymin_varsExpl, ymax_varsExpl))+
      guides(color=guide_colorbar(title.theme = element_text(size=size_legend, angle=0),
                                  label.theme = element_text(size=size_legend, angle=0)))
    
    if(has_drug_targets) {
      gg_cors_bg = gg_cors_bg + 
        labs(fill="Drug type")+
        geom_tile(data=data.frame(x=x_names,y=ymin_cors,Drugtype=drug_targets),aes(x=x,y=y,fill=Drugtype, height=(ymax_cors-ymin_cors)/25), color="white", size=size_tile_border)+
        guides(fill=guide_legend(title.theme = element_text(size=size_legend, angle=0),
                                 label.theme = element_text(size=size_legend, angle=0)))+
        scale_fill_manual(values=drug_target_color)
      
      
      gg_varsExpl_bg = gg_varsExpl_bg + 
        labs(fill="Drug type")+
        guides(fill=guide_legend(title.theme = element_text(size=size_legend, angle=0),
                                 label.theme = element_text(size=size_legend, angle=0)))+
        scale_fill_manual(values=drug_target_color)+
        geom_tile(data=data.frame(x=x_names,y=ymin_varsExpl,Drugtype=drug_targets),aes(x=x,y=y,fill=Drugtype, height=(ymax_varsExpl-ymin_varsExpl)/25), color="white", size=size_tile_border)
      
    } else if(has_manual_cont_annotation) {
      gg_cors_bg = gg_cors_bg + 
        geom_tile(data=df_manual_cont_annotation,aes(x=x_names,y=ymin_cors,fill=val, height=(ymax_cors-ymin_cors)/25), color="white", size=size_tile_border)
      
      gg_varsExpl_bg = gg_varsExpl_bg + 
        geom_tile(data=df_manual_cont_annotation,aes(x=x_names,y=ymin_varsExpl,fill=val, height=(ymax_varsExpl-ymin_varsExpl)/25), color="white", size=size_tile_border)
      
      if(!is.null(manual_cont_annotation_label)) {
        gg_cors_bg = gg_cors_bg + labs(fill=manual_cont_annotation_label)
        gg_varsExpl_bg = gg_varsExpl_bg + labs(fill=manual_cont_annotation_label)
      }
    }
    
  }
  
  if(plot_errorbars) {
    
    df_cors_errorbar2 = df_cors_errorbar %>% 
      group_by(x_names) %>% 
      summarise(ymin=mean(cors)-sd(cors), ymax=mean(cors)+sd(cors)) %>% 
      left_join(df_cors) %>% 
      dplyr::select(-cors)
    
    df_varsExpl_errorbar2 = df_varsExpl_errorbar %>% 
      group_by(x_names) %>% 
      summarise(ymin=mean(varsExpl)-sd(varsExpl), ymax=mean(varsExpl)+sd(varsExpl)) %>% 
      left_join(df_varsExpl) %>% 
      dplyr::select(-varsExpl)
    
    if(plot_bg) {
      ymin_cors = min(df_cors_errorbar2$ymin, min(apply(cors_bg,2, function(x) quantiles_95(x)[5] )))-0.1
      ymin_varsExpl = min(df_varsExpl_errorbar2$ymin, min(apply(varsExpl_bg,2, function(x) quantiles_95(x)[5] )))-0.1
      ymax_cors = max(df_cors_errorbar2$ymax, max(apply(cors_bg,2, function(x) quantiles_95(x)[5] )))+0.1
      ymax_varsExpl = 1.05*max(df_varsExpl_errorbar2$ymax, min(apply(varsExpl_bg,2, function(x) quantiles_95(x)[5] ))) 
      
      gg_errorbar_cors <- gg_cors+ 
        geom_linerange(data=df_cors_errorbar2,aes(ymin=ymin,ymax=ymax, colour=pvals),size=0.7)+
        geom_point(data=df_cors,aes(x=x_names,y=cors,colour=pvals),size=size_marker)+
        scale_colour_gradient(expression(-log[10](P)),high="red",low="blue",limits=c(0,4))+
        coord_cartesian(ylim=c(ymin_cors, ymax_cors))+
        guides(color=guide_colorbar(title.theme = element_text(size=size_legend, angle=0),
                                    label.theme = element_text(size=size_legend, angle=0)))
      
      gg_errorbar_varsExpl <- gg_varsExpl+ 
        geom_linerange(data=df_varsExpl_errorbar2, aes(ymin=ymin,ymax=ymax, colour=pvals),size=0.7)+
        geom_point(data=df_varsExpl,aes(x=x_names,y=varsExpl,colour=pvals),size=size_marker)+
        scale_colour_gradient(expression(-log[10](P)),high="red",low="blue",limits=c(0,4))+
        coord_cartesian(ylim=c(ymin_varsExpl, ymax_varsExpl), expand=F)+
        guides(color=guide_colorbar(title.theme = element_text(size=size_legend, angle=0),
                                    label.theme = element_text(size=size_legend, angle=0)))
      
    } else {
      ymin_cors = min(df_cors_errorbar2$ymin)-0.1
      ymin_varsExpl = min(df_varsExpl_errorbar2$ymin)-0.1
      ymax_cors = max(df_cors_errorbar2$ymax)+0.1
      ymax_varsExpl = max(df_varsExpl_errorbar2$ymax)+0.1
      
      gg_errorbar_cors <- gg_cors+ 
        geom_linerange(data=df_cors_errorbar2,aes(ymin=ymin,ymax=ymax),size=0.7)+
        geom_point(data=df_cors,aes(x=x_names,y=cors),size=size_marker)+
        coord_cartesian(ylim=c(ymin_cors, ymax_cors))+
        guides(color=guide_colorbar(title.theme = element_text(size=size_legend, angle=0),
                                    label.theme = element_text(size=size_legend, angle=0)))
      
      gg_errorbar_varsExpl <- gg_varsExpl+ 
        geom_linerange(data=df_varsExpl_errorbar2, aes(ymin=ymin,ymax=ymax),size=0.7)+
        geom_point(data=df_varsExpl,aes(x=x_names,y=varsExpl),size=size_marker)+
        coord_cartesian(ylim=c(ymin_varsExpl, ymax_varsExpl), expand=F)+
        guides(color=guide_colorbar(title.theme = element_text(size=size_legend, angle=0),
                                    label.theme = element_text(size=size_legend, angle=0)))
    }
    

    
    if(has_drug_targets) {
      gg_errorbar_cors = gg_errorbar_cors + 
        labs(fill="Drug type")+
        geom_tile(data=data.frame(x=x_names,y=ymin_cors,Drugtype=drug_targets),aes(x=x,y=y,fill=Drugtype, height=(ymax_cors-ymin_cors)/25), color="white", size=size_tile_border)+
        guides(fill=guide_legend(title.theme = element_text(size=size_legend, angle=0),
                                 label.theme = element_text(size=size_legend, angle=0)))+
        scale_fill_manual(values=drug_target_color)
      
      gg_errorbar_varsExpl = gg_errorbar_varsExpl +
        labs(fill="Drug type")+
        geom_tile(data=data.frame(x=x_names,y=ymin_varsExpl+0.03,Drugtype=drug_targets),aes(x=x,y=y,fill=Drugtype, height=(ymax_varsExpl-ymin_varsExpl)/25), color="white", size=size_tile_border)+
        guides(fill=guide_legend(title.theme = element_text(size=size_legend, angle=0),
                                 label.theme = element_text(size=size_legend, angle=0)))+
        scale_fill_manual(values=drug_target_color)
    } else if(has_manual_cont_annotation) {
      gg_errorbar_cors = gg_errorbar_cors + 
        geom_tile(data=df_manual_cont_annotation,aes(x=x_names,y=ymin_cors,fill=val, height=(ymax_cors-ymin_cors)/25), color="white", size=size_tile_border)+
        guides(fill=guide_colorbar(title.theme = element_text(size=size_legend, angle=0),
                                   label.theme = element_text(size=size_legend, angle=0)))
      
      gg_errorbar_varsExpl = gg_errorbar_varsExpl + 
        geom_tile(data=df_manual_cont_annotation,aes(x=x_names,y=ymin_varsExpl,fill=val, height=(ymax_varsExpl-ymin_varsExpl)/25), color="white", size=size_tile_border)
      
      if(!is.null(manual_cont_annotation_label)) {
        gg_errorbar_cors = gg_errorbar_cors + labs(fill=manual_cont_annotation_label)
        gg_errorbar_varsExpl = gg_errorbar_varsExpl + labs(fill=manual_cont_annotation_label)
      }
    }
    
    
    if(plot_title) {
      gg_errorbar_cors <- gg_errorbar_cors + ggtitle(title)
      gg_errorbar_varsExpl <- gg_errorbar_varsExpl + ggtitle(title)
    }  
    
    if(plot_bg) {
      
      ymax_mean_cors_cont = 
        ymax_mean_cors[ match(x_names[order(cors,decreasing=T)], ymax_mean_cors$x_names )   ,]
      ymax_mean_cors_cont$x_names = 1:nrow(ymax_mean_cors_cont)
      
      ymax_mean_varsExpl_cont = 
        ymax_mean_varsExpl[ match(x_names[order(varsExpl,decreasing=T)], ymax_mean_varsExpl$x_names )   ,]
      ymax_mean_varsExpl_cont$x_names = 1:nrow(ymax_mean_varsExpl_cont)
      
      p_pos_x = ifelse(p_pos=="ll", 3, length(x_names)-5)
      p_pos_cors_y = ifelse(p_pos=="ll", min(ymax_mean_cors$ymax)-0.05, max(ymax_mean_cors$ymax)+0.05)
      p_pos_varsExpl_y = ifelse(p_pos=="ll", min(ymax_mean_varsExpl$ymax)-0.05, max(ymax_mean_varsExpl$ymax)+0.05)
      
      gg_errorbar_cors <- gg_errorbar_cors +
        geom_crossbar(data=ymax_mean_cors_cont,aes(x=x_names,ymin=ymax, ymax=ymax,y=ymax,group=x_names, fatten=1.5), color="azure4")
      gg_errorbar_varsExpl <- gg_errorbar_varsExpl +
        geom_crossbar(data=ymax_mean_varsExpl_cont,aes(x=x_names,ymin=ymax, ymax=ymax,y=ymax,group=x_names, fatten=1.5), color="azure4")
      
      if(p_pos=="ll" | p_pos=="ur") {
        gg_errorbar_cors = gg_errorbar_cors +annotate(geom="text", x=p_pos_x, y=p_pos_cors_y, hjust=0, label="P=0.05", color="azure4", size=size_axes_text-3)
        gg_errorbar_varsExpl = gg_errorbar_varsExpl + annotate(geom="text", x=p_pos_x, y=p_pos_varsExpl_y, hjust=0, label="P=0.05", color="azure4", size=size_axes_text-3)
      }
    }
    
  }
  
  # Save plots to output list
  if(plot_errorbars) {
    ret$gg_errorbar_cors = gg_errorbar_cors
    ret$gg_errorbar_varsExpl = gg_errorbar_varsExpl
  }
  if(plot_bg) {
    ret$gg_cors_bg = gg_cors_bg
    ret$gg_varsExpl_bg = gg_varsExpl_bg
  }
  
  # avoid too long file names
  argstr_short = input$argstr
  argstr_short = str_replace_all(argstr_short, "UPPERAERODIGESTIVETRACT", "UPDIGTRACT")
  argstr_short = str_replace_all(argstr_short, "HAEMATOPOIETICANDLYMPHOIDTISSUE", "BLOOD")
  
  if(savepdf_cors) {
    save_plot(plot=gg_errorbar_cors,filename=paste(argstr_short,"_cors.pdf",sep=""),base_height=6, base_width=9)
  } 
  if(savepdf_varsExpl) {
    save_plot(plot=gg_errorbar_varsExpl, filename=paste(argstr_short,"_varsExpl.pdf",sep=""),base_height=6, base_width=9)
  }
  if(savepdf_cors_bg & plot_bg) {
    save_plot(plot=gg_cors_bg,filename=paste(argstr_short,"_bg_cors.pdf",sep=""),base_height=6, base_width=9)
  }
  if(savepdf_varsExpl_bg & plot_bg) {
    save_plot(plot=gg_varsExpl_bg,filename=paste(argstr_short,"_bg_varsExpl.pdf",sep=""),base_height=6, base_width=9)
  }
  return(ret)
}


####################################
### PLOT OUT-CORS-HEATMAP ##########
####################################

get_plotcorsheatmap_width = function(out_list) {
  max_drugname_length = max(unlist(map(names(out_list[[1]]$cors), function(x) str_length(x))))
  n_heatmap_cols = length(out_list)
  return(2.3*(max_drugname_length+1.7*n_heatmap_cols)/44.0+2.2)
}

plot_corsheatmap = function(outlist, labels, valVar="cors", plot_legend=F, line_break=T, fontsize_y=size_axes_text-1, colorbar_height=3.0) {
  shared_drugs = reduce( lapply(outlist, function(o) names(o$cors)), intersect)
  df_wide = data.frame(sapply(outlist, function(o) o[[valVar]][names(o$cors) %in% shared_drugs]))
  colnames(df_wide) = labels
  
  max_drugname_length = max(unlist(map(names(outlist[[1]]$cors), function(x) str_length(x))))
  n_heatmap_cols = length(outlist)
  
  # take -log10 of p-values
  if(grepl("pvals",valVar))
    df_wide = -log10(df_wide)
  
  cluster_order = hclust(dist(df_wide))$order
  df_tidy = df_wide %>% 
    mutate(drug = rownames(df_wide)) %>% 
    gather(Type, val,-drug)
  
  df_tidy$drug = ordered(df_tidy$drug, levels=rownames(df_wide)[cluster_order])
  df_tidy$Type = ordered(df_tidy$Type, levels=colnames(df_wide))
  
  if(line_break)
    labels_break = gsub("\\+", "\\+\n",labels)
  else
    labels_break = labels
  
  gg = ggplot(df_tidy, aes(x=Type, y=drug))+
    geom_tile(aes(fill=val), color="white", size=size_tile_border)+
    theme(axis.line=element_blank(),
          axis.ticks=element_blank(),
          axis.title.x=element_blank(),
          axis.title.y=element_blank(),
          legend.text=element_text(size=size_axes_title),
          legend.title=element_text(size=size_axes_title+2),
          axis.text.x = element_text(colour = "black", size=size_axes_title-1, angle=-90, vjust=0.5, hjust=0, margin=margin(t=-1)),
          axis.text.x.top = element_text(vjust = 0, margin=margin(b=-1)),
          axis.text.y = element_blank())+
    scale_x_discrete(position="bottom", labels=labels_break)+
    guides(fill=guide_colorbar(title.position="top", 
                               title.theme = element_text(size=size_axes_title, angle=0),
                               label.theme = element_text(size=size_axes_title, angle=0), 
                               barheight=colorbar_height, barwidth=0.40, title.vjust = 0, title.hjust = 0, label.vjust = 0))
  
  if(valVar=="cors") {
    gg = gg+scale_fill_gradientn(expression(rho[s]), 
                                 limit=c(0.15,0.7),
                                 values=rescale(c(0.15, 0.299, 0.3, 0.5, 0.7)),
                                 breaks=c(0.15, 0.3, 0.5),
                                 labels=c("<0.3", "0.3", "0.5"),
                                 colors=c("gray", "gray", "white", "#FF8080", "#FF0000"),
                                 oob = scales::squish)
  } else if(valVar=="varsExpl") {
    gg = gg+scale_fill_gradientn(expression(R^2), 
                                 limit=c(-0.15,0.4),
                                 values=rescale(c(-0.15, -0.01, 0, 0.15, 0.30)),
                                 breaks=c(0, 0.15, 0.30),
                                 labels=c("<0.15", "0.15", "0.30"),
                                 colors=c("gray", "gray", "white", "#FF8080", "#FF0000"),
                                 oob = scales::squish)
  } else if(valVar=="pvals_cors" | valVar=="pvals_varsExpl") {
    gg = gg + scale_fill_gradientn(expression(-log[10](P)), 
                                   limit=c(0,4),
                                   values=rescale(c(0, 1.3-0.01, 1.3, 4 )),
                                   colors=c("gray","gray", "white", "#FF0000"),
                                   oob = scales::squish)
  } else if(valVar=="pvals_cors_adj" | valVar=="pvals_varsExpl_adj") {
    gg = gg + scale_fill_gradientn(expression(-log[10](Padj)), 
                                   limit=c(0,4),
                                   values=rescale(c(0, 1.3-0.01, 1.3, 4 )),
                                   colors=c("gray","gray", "white", "#FF0000"),
                                   oob = scales::squish)
  } 
  
  colorbar_legend = get_legend(gg)
  gg = gg+guides(fill=F)
  
  drug_target_lab = drug_type_label(outlist[[1]]$drug_target[names(outlist[[1]]$cors) %in% shared_drugs])
  
  df_row_annotate = data.frame(drug=ordered(rownames(df_wide), levels=rownames(df_wide)[cluster_order]), target=outlist[[1]]$drug_target[names(outlist[[1]]$cors) %in% shared_drugs])
  gg_annotate = ggplot(df_row_annotate, aes(x="Drug type", y=drug, fill=target)) +
    geom_tile(color="white", size=size_tile_border)+
    theme(axis.line=element_blank(),
          axis.ticks=element_blank(),
          axis.title.x=element_blank(),
          axis.title.y=element_blank(),
          legend.text=element_text(size=size_axes_title),
          legend.title=element_text(size=size_axes_title+2),
          axis.text.x = element_text(colour = "black", size=size_axes_title-1, angle=-90, vjust=0.5, margin=margin(t=-1)),
          axis.text.y = element_blank(),
          plot.margin = unit(c(0, 0, 0, -0.2), "cm"))+
    labs(fill="Drug type")+
    scale_fill_manual(values=drug_target_lab$drug_target_color)+
    scale_y_discrete(position="right")+
    scale_x_discrete(position="bottom")
  
  gg_annotate_leg = get_legend(gg_annotate)
  gg_annotate = gg_annotate + theme(legend.position = "none")
  
  
  gg_annotate_axistext = ggplot(df_row_annotate, aes(x="Drug type", y=drug, fill=target)) +
    theme(axis.line=element_blank(),
          axis.ticks=element_blank(),
          axis.title.x=element_blank(),
          axis.title.y=element_blank(),
          axis.text.x = element_blank(),
          axis.text.y = element_text(colour = "black", size=fontsize_y, angle=0),
          plot.margin = unit(c(0, 0, 0, -0.3), "cm"),
          legend.position = "none")+
    labs(fill="")+
    scale_fill_manual(values=drug_target_lab$drug_target_color)+
    scale_y_discrete(position="right")+
    scale_x_discrete(position="bottom")
  
  
  plots = align_plots(gg, gg_annotate, gg_annotate_axistext, align="h")
  
  if(length(outlist)>10) {
    q = 1.7*n_heatmap_cols/(1.7*n_heatmap_cols+max_drugname_length)
  } else {
    q = 0.6
  }
  
  fig = plot_grid(plots[[1]], plots[[2]], plots[[3]], rel_widths = c(q, (1-q)*2.0/max_drugname_length, 1-q), ncol=3)
  
  if(plot_legend) {
    fig = plot_grid(fig, NULL, ncol=2, rel_widths = c(0.60, 0.40))
    fig = fig + draw_plot(gg_annotate_leg, x=0.65, y=0.3)+draw_plot(colorbar_legend, x=0.65, y=0.0)
  }
  return(list(fig=fig, leg_ann=gg_annotate_leg, leg_colorbar=colorbar_legend))
  
}


####################################
###### PLOT-DRUGX-HEATMAP ##########
####################################
plot_drugX_heatmap = function(df, drug_targets, gradient="cors", axis.text.y=TRUE, draw_colorbar_legend=T, draw_annotation_legend=T, parse_ylab=F) {
  
  drug_targets_lab = drug_type_label(drug_targets$drug_targets)
  
  cluster_order_rows = hclust(dist(df), method="complete")$order
  cluster_cols = hclust(dist(t(df)), method="complete")
  cluster_order_cols = cluster_cols$order
  
  # heatmap
  df_tidy = df %>%
    as_data_frame %>%
    mutate(y=rownames(df)) %>% 
    gather(key="Drug", value="val",-y) %>% 
    mutate(Drug=ordered(Drug, levels=colnames(df)[cluster_order_cols])) %>% 
    mutate(y=ordered(y, levels=rownames(df)[cluster_order_rows]))
  
  gg = ggplot(df_tidy, aes(x=Drug, y=y))+ 
    geom_tile(aes(fill=val),  color="white", size=size_tile_border)+
    theme(axis.line=element_blank(),
          axis.ticks=element_blank(),
          axis.title.x=element_blank(),
          axis.title.y=element_blank(),
          legend.text=element_text(size=size_axes_title),
          legend.title=element_text(size=size_axes_title+2),
          axis.text.x = element_text(size=size_axes_title-1, angle=-90, vjust=0.5, hjust=0 ),
          #axis.text.y = element_text(size=size_axes_title-3, hjust=1),
          axis.text.y = element_blank(),
          legend.position = "right")+
    labs(fill="")+
    scale_x_discrete(position="bottom", expand=c(0,0))
  
  if(parse_ylab)
    gg = gg + scale_y_discrete(labels=rppa_pub_label(levels(df_tidy$y)))
  
  if(axis.text.y)
    gg = gg + theme(axis.text.y = element_text(size=size_axes_title-3, hjust=1))
  
  if(gradient=="cors") {
    gg = gg + scale_fill_gradientn(expression(rho[s]), 
                                   limit=c(-0.5,0.5),
                                   breaks=c(-0.5, -0.25, 0, 0.25, 0.5),
                                   colors=c("dodgerblue4", "dodgerblue1", "white", "#FF8080", "#FF0000"),
                                   oob = scales::squish)+
      labs(fill=expression(rho[s]))
  } else if(gradient=="log10Padj") {
    gg = gg + scale_fill_gradientn(expression(-log[10](Padj)), 
                                   limit=c(0,2),
                                   values=rescale(c(0, 1.3-0.01, 1.3, 2 )),
                                   colors=c("gray","gray", "white", "#FF0000"),
                                   oob = scales::squish)
  } else if(gradient=="log10P") {
    gg = gg + scale_fill_gradientn(expression(-log[10](P)), 
                                   limit=c(0,3),
                                   values=rescale(c(0, 1.3-0.01, 1.3, 3 )),
                                   colors=c("gray","gray", "white", "#FF0000"),
                                   oob = scales::squish)
  } else if(gradient=="PLScoefs") {
    gg = gg + scale_fill_gradientn("PLS coefficient", 
                                   limit=c(-3, 3),
                                   breaks=c(-3, 0, 3),
                                   colors=c("blue","white", "#FF0000"), # FF8080", 
                                   oob = scales::squish)+
      guides(fill=guide_colorbar(title.position="left", title.theme = element_text(size=8, angle=90),
                                 label.theme = element_text(size=8, angle=0), 
                                 barheight=3.5, barwidth=0.45))
  }
  
  
  gg_leg = get_legend(gg)
  gg = gg + theme(legend.position="none")
  
  # drug annotation
  df_ann = data.frame(Drug=colnames(df)[cluster_order_cols],
                      `Drug type`=drug_targets_lab$drug_targets[cluster_order_cols], 
                      check.names = F)
  df_ann$Drug = ordered(df_ann$Drug, levels=df_ann$Drug)
  
  gg_ann = ggplot(df_ann, aes(x=Drug,y=0))+
    geom_tile(aes(fill=`Drug type`), color="white", size=size_tile_border)+
    theme(axis.line=element_blank(),
          axis.ticks=element_blank(),
          axis.title.x=element_blank(),
          axis.title.y=element_blank(),
          axis.text.x = element_blank(),
          #axis.text.x.top = element_text(margin=margin(t=-1)),
          axis.text.y = element_blank(),
          axis.line.y = element_blank(),
          legend.text=element_text(size=size_axes_title),
          legend.title=element_text(size=size_axes_title+2),
          plot.margin = unit(c(0, 0, -0.2, 0), "cm"))+
    scale_fill_manual(values=drug_targets_lab$drug_target_color)+
    scale_x_discrete(position="bottom", expand=c(0,0))
  
  gg_ann_leg = get_legend(gg_ann)
  gg_ann = gg_ann + theme(legend.position="none")
  
  # dendrogram
  dend = as.dendrogram(cluster_cols)
  dend_data = dendro_data(dend)
  
  dg = ggplot(segment(dend_data))+
    geom_segment(aes(x=x, y=y, xend=xend, yend=yend), size=0.35)+
    #coord_flip() + 
    #scale_y_reverse() +
    scale_y_continuous(expand=c(0, 0))+
    scale_x_continuous(expand=c(1/length(dend_data$labels$label)/2, 0))+
    theme_dendro()+
    theme(plot.margin = unit(c(0.1, 0, -0.8, 0), "cm"))
  
  gg_ann_dg = plot_grid(dg, gg_ann, gg, nrow=3, align="v", axis="y", rel_heights = c(0.03, 0.015, 0.90))
  
  if(draw_colorbar_legend | draw_annotation_legend) {
    gg_ret = plot_grid(gg_ann_dg, NULL, ncol=2, rel_widths = c(0.80, 0.25))
    if(draw_colorbar_legend)
      gg_ret = gg_ret + draw_plot(gg_leg, x=0.80, y=-0.05)
    if(draw_annotation_legend)
      gg_ret = gg_ret + draw_plot(gg_ann_leg, x=0.80, y=0.3)
  } else {
    gg_ret = gg_ann_dg
  }
  
  
  return(gg_ret)
  
  
}



####################################
##### PLOT COR-VS-COR SCATTER ######
####################################

plot_corcor_scatter = function(df, xval, yval, valVar="cor", color_tissue=F, abline=T) {
  if(color_tissue) {
    df = df %>% 
      dplyr::select(datatype, drug, !! sym(valVar), tissue) %>% 
      spread(key=datatype, value=!! sym(valVar))
  } else {
    df = df %>% 
      dplyr::select(datatype, drug, !! sym(valVar)) %>% 
      spread(key=datatype, value=!! sym(valVar))
  }
  
  cor_lab = paste("rho[s] == '", sprintf('%.2f',cor(df[,xval], df[,yval], use="complete.obs", method="spearman")),"'",sep="")
  
  if(color_tissue)
    gg =  ggplot(df, aes_string(x=xval, y=yval, color="tissue"))
  else
    gg =  ggplot(df, aes_string(x=xval, y=yval))
  
  cor_lab_x = min(df[,xval], na.rm=T)+0.07
  cor_lab_y = max(df[,yval], na.rm=T)-0.05
  
  gg = gg +
    geom_point()+
    xlab(parse(text=(paste("rho[",firstlow(xval),"]"))))+
    ylab(parse(text=(paste("rho[",firstlow(yval),"]"))))+
    theme(axis.title = element_text(size=size_axes_title),
          axis.text = element_text(size=size_axes_text), 
          legend.title = element_blank(),
          legend.text = element_text(size=size_axes_text),
          legend.key.size = unit(size_axes_text, "points"),
          #plot.margin = unit(c(0,0,0,0), "cm"),
          panel.background = element_rect(colour = "black", fill=NA, size=1, linetype="solid"))+
    annotate("text", x=cor_lab_x, y=cor_lab_y, label=cor_lab, parse=T, size=3.5)
  
  if(abline)
    gg = gg + geom_abline(intercept = 0, slope = 1, color="gray")
  
  return(gg)
  
}